/*******************************************************************************
 * Copyright (C) 2006 Ping Identity Corporation All rights reserved.
 * 
 * This software is licensed under the Open Software License v2.1 (OSL v2.1).
 * 
 * A copy of this license has been provided with the distribution of this
 * software. Additionally, a copy of this license is available at:
 * http://opensource.org/licenses/osl-2.1.php
 *  
 ******************************************************************************/
using System;
using System.Collections;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.SessionState;
using System.IO;
using Com.PingIdentity.PingFederate.SampleApp.SampleAppUtility;

namespace Com.PingIdentity.PingFederate.SampleApp.IdpSample.cs.main {

	/// <summary>
	/// This is the main handler of the IdP Sample Application
	/// and handles local login
	/// </summary>
	public class MainHandler : IHttpHandler, IRequiresSessionState {
		
		public void ProcessRequest(HttpContext context) {
			
			HttpRequest  request     = context.Request;
			HttpResponse response    = context.Response;
			HttpSessionState session = context.Session;

			//get properties from Idp config file
			Hashtable idpConfigTable;
			if(session[Constants.IDP_CONFIG_TABLE] == null) {
				idpConfigTable = GetIdpConfigProps(context);
				session[Constants.IDP_CONFIG_TABLE] = idpConfigTable;
			}
			else {
				idpConfigTable = (Hashtable)session[Constants.IDP_CONFIG_TABLE];
			}
			context.Trace.Warn("idpConfigTable has been loaded");

			//get properties from Idp connections file
			ArrayList idpConnectionsList;
			if(session[Constants.IDP_CONNECT_LIST] == null) {
				idpConnectionsList = GetIdpConnectionsProps(context);
				session[Constants.IDP_CONNECT_LIST] = idpConnectionsList;
			}
			else {
				idpConnectionsList = (ArrayList)session[Constants.IDP_CONNECT_LIST];
			}
			context.Trace.Warn("idpConnectionsList has been loaded");

			Links samlLinks = new Links(idpConfigTable, request);

			//create URL to Login Page
			String loginPage = samlLinks.getIdpLoginPageUrl(Constants.LOCAL_TYPE);

			//Get user attributes from session
			Hashtable userInfo = (Hashtable)session[Constants.USER_INFO];
			if((userInfo == null) || (userInfo.Count <= 0)) {
				context.Trace.Warn("userInfo = null, redirect to loginPage");
				response.Redirect(loginPage);
				return;
			}

			//check the logout case
			String cmd = request.QueryString.Get(Constants.PARAM_CMD);
			if( cmd != null && cmd.Equals(Constants.CMD_LOGOUT)) {
				String resumePath = request.QueryString.Get(Constants.RESUME_PATH);
				FormsAuthentication.SignOut();
				session.Abandon();
				context.Trace.Warn("logout case: cmd = logout");
				if(resumePath != null) {
					resumePath = samlLinks.getResumePathUrl(resumePath); //create full path
					response.Redirect(resumePath);
				}
				else {
					response.Redirect(loginPage);
				}
				return;
			}

			// Redirect to Idp application protected page
			String mainpage = samlLinks.getIdpMainPageUrl();
			response.Redirect(mainpage);
		}

		/// <summary>
		/// Reads general configuration properties XML file for the
		/// IDP Sample Application and converts it to Hastable object
		/// </summary>
		/// <param name="context">current HttpContext</param>
		/// <returns>Hashtable of user general configuration properties</returns>
		private Hashtable GetIdpConfigProps(HttpContext context) {

			//read properties from Idp config XML file
			String idpConfig  = ConfigurationSettings.AppSettings["config-file"];
			String configPath = context.Server.MapPath("config/" + idpConfig);

			XmlUtility xmlUtility = new XmlUtility();
			Hashtable idpConfigTable = xmlUtility.GetConfigProps(configPath);
			if(idpConfigTable == null) {
				String errorMessage = "Error in " + idpConfig;
				context.Response.Redirect(Constants.WEB_PREFIX + Constants.ERROR_PAGE + errorMessage);
			}
			return idpConfigTable;
		}

		/// <summary>
		/// Reads connection properties XML file for specifying Sp Connections
		/// and converts it to ArrayList of Hashtables with user SP connections
		/// </summary>
		/// <param name="context">current HttpContext</param>
		/// <returns>ArrayList of user connection properties</returns>
		private ArrayList GetIdpConnectionsProps(HttpContext context) {

			//read properties from Idp connections XML file
			String idpConnections  = ConfigurationSettings.AppSettings["connections-file"];
			String connectionsPath = context.Server.MapPath("config/" + idpConnections);

			if (File.Exists(connectionsPath)) 
			{
				XmlUtility xmlUtility = new XmlUtility();
				ArrayList connectionsList = xmlUtility.GetPropsFromXmlFile(connectionsPath);
				if(connectionsList == null || connectionsList.Count == 0) 
				{
					String errorMessage = "Error in " + idpConnections;
					context.Response.Redirect(Constants.WEB_PREFIX + Constants.ERROR_PAGE + errorMessage);
				}
				return connectionsList;
			}
			else
			{
				return null;
			}
		}

		public bool IsReusable {
			get { return true; }
		}
	}
}