/*******************************************************************************
 * Copyright (C) 2006 Ping Identity Corporation All rights reserved.
 * 
 * This software is licensed under the Open Software License v2.1 (OSL v2.1).
 * 
 * A copy of this license has been provided with the distribution of this
 * software. Additionally, a copy of this license is available at:
 * http://opensource.org/licenses/osl-2.1.php
 *  
 ******************************************************************************/
using System;
using System.Collections;
using System.Configuration;
using System.IO;
using System.Web;
using System.Web.Security;
using System.Web.SessionState;
using Com.PingIdentity.Adapters.Pftoken.Core;
using Com.PingIdentity.PingFederate.SampleApp.SampleAppUtility;

namespace Com.PingIdentity.PingFederate.SampleApp.SpSample.cs.slo {

	/// <summary>
	/// SLOSpHandler is the logout service of SP Sample application. It starts the SLO 
	/// process for SP-initiated SLO and performs local logout for IdP-initiated SLO.
	/// </summary>
	public class SLOSpHandler : IHttpHandler, IRequiresSessionState {

		private Links samlLinks;

		public void ProcessRequest(HttpContext context) {

			HttpRequest  request     = context.Request;
			HttpResponse response    = context.Response;
			HttpSessionState session = context.Session;

			//get properties from Sp config file
			Hashtable spConfigTable;
			if(session[Constants.SP_CONFIG_TABLE] == null) {
				spConfigTable = GetSpConfigProps(context);
				session[Constants.SP_CONFIG_TABLE] = spConfigTable;
			}
			else {
				spConfigTable = (Hashtable)session[Constants.SP_CONFIG_TABLE];
			}
			context.Trace.Warn("spConfigTable has been loaded");

			samlLinks = new Links(spConfigTable, request);

			// Create url for redirect after performing logout
			String returnURL = createRedirectURL(request);
			context.Trace.Warn("redirectURL = " + returnURL);

			//get sessionID from the request
			String userSessionID = request.QueryString.Get(Constants.SESSION_ID_PARAM);

			if(userSessionID == null) 
			{
				context.Trace.Warn("userSessionID = null, redirect to " + returnURL);
				response.Redirect(returnURL);
				return;
			}
			else
			{
				returnURL = setLogoutPFToken(context, returnURL, userSessionID, true);

				FormsAuthentication.SignOut();
				session.Abandon();
			}

			// Redirect to url created earlier
			response.Redirect(returnURL);
		}

		/// <summary>
		/// createRedirectURL method checks to see if the request contains resume path. This
		/// indicates that the request came from PingFederate server and the resume 
		/// path will be used for the response url. If there is no resume path, the 
		/// response url is obtained from the servlet configuration and it is the url
		/// to the SLO service for Sp Sample Application
		/// </summary>
		/// <param name="request">current user request</param>
		/// <returns>url to redirect</returns>
		private String createRedirectURL(HttpRequest request) {

			// Get resume path from request
			String urlToSpApp = request.QueryString.Get(Constants.RESUME_PATH);
		
			if(urlToSpApp == null) {
				// There is no resume path so use the url to Sp SLO service
			
				// This is the link to start SP-initiated SLO
				urlToSpApp = samlLinks.getSpSLOLink();			
			}
			else {
				urlToSpApp = samlLinks.getResumePathUrl(urlToSpApp);  //create full path
			}
			return urlToSpApp;
		}
	
		/// <summary>
		/// setLogoutPFToken method uses the Java Integration Kit API to create an 
		/// instance of Sp Agent and construct the logout PFTOKEN
		/// </summary>
		/// <param name="context">current HttpContext</param>
		/// <param name="redirectURL">url for redirect after performing logout</param>
		/// <param name="userSessionID">current session Id</param>
		/// <param name="logoutSuccessful">flag indicating successful logout</param>
		/// <returns>URL with pftoken logout parameter</returns>
		private String setLogoutPFToken(HttpContext context, String url, String userSessionID,
				bool logoutSuccessful) {

			String returnUrl = url;
			HttpRequest  request  = context.Request;
			HttpResponse response = context.Response;
			HttpSessionState session = context.Session;

			//get properties from Sp config file
			Hashtable spConfigTable;
			if(session[Constants.SP_CONFIG_TABLE] == null) 
			{
				spConfigTable = GetSpConfigProps(context);
				session[Constants.SP_CONFIG_TABLE] = spConfigTable;
			}
			else 
			{
				spConfigTable = (Hashtable)session[Constants.SP_CONFIG_TABLE];
			}
			context.Trace.Warn("spConfigTable has been loaded");
		
			//read properties from Sp pfagent-properties file
			String spProps   = ConfigurationSettings.AppSettings["pfagent-properties"];
			String propsPath = context.Server.MapPath("config/" + spProps);

			//get password
			String password = (String)spConfigTable[Constants.PASSWORD];

			//get holder name
			String holderName = (String)spConfigTable[Constants.HOLDER_NAME];

			// Create an instance of PFTokenSpAgent
			PftokenAgent spAgent = new PftokenAgent(password,holderName);		

 			// Set userSessionID in Sp Agent
			spAgent.SessionId = userSessionID;

 			// Create logout PFTOKEN
			returnUrl = spAgent.ConstructLogoutPFToken(request, response, url, logoutSuccessful);
			return returnUrl;
		}
	
		/// <summary>
		/// Reads general configuration properties XML file for the
		/// SP Sample Application and converts it to Hastable object
		/// </summary>
		/// <param name="context">current HttpContext</param>
		/// <returns>Hashtable of user general configuration properties</returns>
		private Hashtable GetSpConfigProps(HttpContext context) {

			//read properties from Sp config XML file
			String spConfig  = ConfigurationSettings.AppSettings["config-file"];
			String configPath = context.Server.MapPath("config/" + spConfig);

			XmlUtility xmlUtility = new XmlUtility();
			Hashtable spConfigTable = xmlUtility.GetConfigProps(configPath);
			if(spConfigTable == null) {
				String errorMessage = "Error in " + spConfig;
				context.Response.Redirect(Constants.WEB_PREFIX + Constants.ERROR_PAGE + errorMessage);
			}
			return spConfigTable;
		}

		public bool IsReusable {
			get { return true; }
		}
	}
}